@@ -2,7 +2,7 @@
 //
 //  RcppTOML -- Rcpp bindings to TOML via cpptoml
 //
-//  Copyright (C) 2015 - 2017  Dirk Eddelbuettel 
+//  Copyright (C) 2015 - 2018  Dirk Eddelbuettel
 //
 //  This file is part of RcppTOML
 //
@@ -92,10 +92,14 @@ inline time_t local_timegm(struct tm *tm) {
 #endif
 }
 
-SEXP getValue(const std::shared_ptr<cpptoml::base>& base) {
+SEXP getValue(const std::shared_ptr<cpptoml::base>& base, bool escape=true) {
     if (auto v = base->as<std::string>()) {
-        std::string s(escapeString(v->get()));
-        return Rcpp::wrap(s);
+        std::string s(v->get());
+        if (escape) {
+            s = escapeString(s);
+        }
+        Rcpp::String se(s, CE_UTF8);
+        return Rcpp::wrap(se);
     } else if (auto v = base->as<int64_t>()) {
         std::int64_t s(v->get());
         int t = static_cast<int>(s); // we need int for wrap to work
@@ -119,7 +123,7 @@ SEXP getValue(const std::shared_ptr<cpptoml::base>& base) {
         tm.tm_hour = s.hour;
         tm.tm_min  = s.minute;
         tm.tm_sec  = s.second;
-        time_t tt = local_timegm(&tm); 
+        time_t tt = local_timegm(&tm);
         //tt = tt - s.hour_offset*60*60 - s.minute_offset*60;
         Rcpp::DatetimeVector dt(1, "UTC");
         dt[0] = tt + s.microsecond * 1.0e-6;
@@ -133,11 +137,23 @@ SEXP getValue(const std::shared_ptr<cpptoml::base>& base) {
         tm.tm_hour = s.hour;
         tm.tm_min  = s.minute;
         tm.tm_sec  = s.second;
-        time_t tt = local_timegm(&tm); 
+        time_t tt = local_timegm(&tm);
         tt = tt - s.hour_offset*60*60 - s.minute_offset*60;
         Rcpp::DatetimeVector dt(1, "UTC");
         dt[0] =  tt + s.microsecond * 1.0e-6;
         return Rcpp::wrap(dt);
+    } else if (auto v = base->as<cpptoml::local_time>()) {
+        cpptoml::local_time t(v->get());
+        char txt[32];
+        if (t.microsecond != 0) {
+            snprintf(txt, 31, "%02d:%02d:%02d.%d",
+                     t.hour, t.minute, t.second, t.microsecond);
+        } else {
+            snprintf(txt, 31, "%02d:%02d:%02d",
+                     t.hour, t.minute, t.second);
+        }
+        std::string s(txt);
+        return Rcpp::wrap(s);
     } else {
         Rcpp::warning("Unparsed value, returning null");
         return R_NilValue;
@@ -152,7 +168,7 @@ void printArray(std::ostream& o, cpptoml::array& arr) {
             printArray(o, *(*it)->as_array());
         else
             printValue(o, *it);
-        
+
         if (++it != arr.get().end())
             o << ", ";
     }
@@ -161,7 +177,7 @@ void printArray(std::ostream& o, cpptoml::array& arr) {
 
 SEXP collapsedList(Rcpp::List ll) {
     if (ll.length() == 0) return R_NilValue;
-    Rcpp::List::iterator it = ll.begin(); 
+    Rcpp::List::iterator it = ll.begin();
     switch(TYPEOF(*it)) {
         case REALSXP: {
             Rcpp::NumericVector v(ll.begin(), ll.end());
@@ -204,20 +220,20 @@ SEXP collapsedList(Rcpp::List ll) {
     return ll;
 }
 
-SEXP getArray(const cpptoml::array& arr) {
+SEXP getArray(const cpptoml::array& arr, bool escape=true) {
     Rcpp::StretchyList sl;
     bool nonested = true;       // ie no embedded array
     auto it = arr.get().begin();
     while (it != arr.get().end()) {
         if ((*it)->is_array()) {
-            sl.push_back(getArray(*(*it)->as_array())); 
+            sl.push_back(getArray(*(*it)->as_array()));
             nonested = false;
         } else {
-            sl.push_back(getValue(*it));
+            sl.push_back(getValue(*it, escape));
             nonested = true;
         }
         it++;
-    } 
+    }
     if (nonested) {
         return collapsedList(Rcpp::as<Rcpp::List>(sl));
     } else {
@@ -226,7 +242,7 @@ SEXP getArray(const cpptoml::array& arr) {
 }
 
 
-SEXP getTable(const std::shared_ptr<cpptoml::table>& t, bool verbose=false) {
+SEXP getTable(const std::shared_ptr<cpptoml::table>& t, bool verbose=false, bool escape=true) {
     Rcpp::StretchyList sl;
     for (auto & p : *t) {
         if (p.second->is_table()) {
@@ -239,14 +255,14 @@ SEXP getTable(const std::shared_ptr<cpptoml::table>& t, bool verbose=false) {
                 Rcpp::Rcout << "Array: " << p.first << std::endl;
                 printArray(Rcpp::Rcout, *ga);
             }
-            sl.push_back(Rcpp::Named(p.first) = getArray(*ga)); 
+            sl.push_back(Rcpp::Named(p.first) = getArray(*ga));
         } else if (p.second->is_value()) {
             if (verbose) {
                 Rcpp::Rcout << "Value: " << p.first << "\n  :";
                 printValue(Rcpp::Rcout, p.second);
                 Rcpp::Rcout << std::endl;
             }
-            sl.push_back(Rcpp::Named(p.first) = getValue(p.second)); 
+            sl.push_back(Rcpp::Named(p.first) = getValue(p.second, escape));
         } else if (p.second->is_table_array()) {
             if (verbose) Rcpp::Rcout << "TableArray: " << p.first << std::endl;
             Rcpp::StretchyList l;
@@ -260,7 +276,7 @@ SEXP getTable(const std::shared_ptr<cpptoml::table>& t, bool verbose=false) {
             sl.push_back(Rcpp::Named(p.first) = Rcpp::as<Rcpp::List>(l));
         } else {
             if (verbose) Rcpp::Rcout << "Other: " << p.first << std::endl;
-            sl.push_back(p.first); 
+            sl.push_back(p.first);
         }
     }
     return Rcpp::as<Rcpp::List>(sl);
@@ -271,7 +287,8 @@ SEXP getTable(const std::shared_ptr<cpptoml::table>& t, bool verbose=false) {
 Rcpp::List tomlparseImpl(const std::string input,
                          bool verbose=false,
                          bool fromfile=true,
-                         bool includize=false) {
+                         bool includize=false,
+                         bool escape=true) {
 
     if (fromfile && access(input.c_str(), R_OK)) {
         Rcpp::stop("Cannot read given file '" + input + "'.");
@@ -292,11 +309,11 @@ Rcpp::List tomlparseImpl(const std::string input,
         cpptoml::parser p(strstream);
         g = p.parse();
     }
-    
+
     if (verbose) {
-        Rcpp::Rcout << "<default print method>\n" 
+        Rcpp::Rcout << "<default print method>\n"
                     << (*g)
-                    << "</default print method>\n" 
+                    << "</default print method>\n"
                     << std::endl;
     }
 
@@ -324,7 +341,7 @@ Rcpp::List tomlparseImpl(const std::string input,
         } else if (p.second->is_array()) {
             auto ga = std::dynamic_pointer_cast<cpptoml::array>(p.second);
             if (verbose) Rcpp::Rcout << "Array: " << p.first << std::endl;
-            sl.push_back(Rcpp::Named(p.first) = getArray(*ga)); 
+            sl.push_back(Rcpp::Named(p.first) = getArray(*ga));
 
         } else if (p.second->is_value()) {
             if (verbose) {
@@ -332,13 +349,12 @@ Rcpp::List tomlparseImpl(const std::string input,
                 printValue(Rcpp::Rcout, p.second);
                 Rcpp::Rcout << std::endl;
             }
-            sl.push_back(Rcpp::Named(p.first) = getValue(p.second)); 
-            
+            sl.push_back(Rcpp::Named(p.first) = getValue(p.second, escape));
         } else {
             if (verbose) Rcpp::Rcout << "Other: " << p.first << std::endl;
-            sl.push_front(p.first); 
+            sl.push_front(p.first);
         }
     }
-    
+
     return Rcpp::as<Rcpp::List>(sl);
 }
